/*	xmms ao output plugin
 *	Copyright (C) 2003 Levi Ramsey
 *
 *	This program is Free Software and licensed under the GNU GPL
 *
 *	Description: Outputs audio from xmms to libao2.  This enables automatic
 *		detection of sound system to use (OSS, ALSA, esd, aRts, etc.)
 *
 *	Based on Galex Yen's esd output plugin
 *
 *	Contains code (C) 1998-1999 Mikael Alm, Olle Hallnas, Thomas Nilsson,
 *		and 4Front Technologies
 */

#include "aoout.h"
#include <xmms/configfile.h>
#include <string.h>
#include <stdio.h>

static GtkWidget *conf_win;
static GtkWidget *buf_en_check;
static GtkWidget *buf_size_spin;
static GtkWidget *buf_chunk_spin;
static GtkWidget *file_en_check;
static GtkWidget *file_over_check;
static GtkWidget *file_dir_entry;
static GtkWidget *file_browser=NULL;
static GtkWidget *effect_prevol_check;
static GtkWidget *effect_file_check;
static char *file_type;

static GtkWidget *aoout_config_buf_frame(void);
static GtkWidget *aoout_config_file_frame(void);
static GtkWidget *aoout_config_effect_frame(void);

static void aoout_config_ok(GtkWidget *widget, gpointer data);
static void aoout_config_apply(GtkWidget *widget, gpointer data);
static void aoout_config_cancel(GtkWidget *widget, gpointer data);
static void aoout_config_browse(GtkWidget *widget, gpointer data);
static void aoout_config_dirupd8(gchar *dir);
static void aoout_configure_ftsel(GtkWidget *widget, gpointer data);

void aoout_configure(void) {
	GtkWidget *vbox;
	GtkWidget *but_box, *okbutton, *applybutton, *cancelbutton;

	if (conf_win) {
		gdk_window_raise(conf_win->window);
		return;
	}

	conf_win=gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_signal_connect(GTK_OBJECT(conf_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &conf_win);
	gtk_window_set_title(GTK_WINDOW(conf_win), _("xmmsao configuration"));
	gtk_window_set_policy(GTK_WINDOW(conf_win), FALSE, FALSE, FALSE);
	gtk_container_border_width(GTK_CONTAINER(conf_win), 4);

	vbox=gtk_vbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(conf_win), vbox);

	gtk_box_pack_start(GTK_BOX(vbox), aoout_config_buf_frame(), TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), aoout_config_file_frame(), TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), aoout_config_effect_frame(), TRUE, TRUE, 0);

	but_box=gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), but_box, TRUE, TRUE, 2);

	okbutton=gtk_button_new_with_label("OK");
	gtk_signal_connect(GTK_OBJECT(okbutton), "clicked", GTK_SIGNAL_FUNC(aoout_config_ok), NULL);
	gtk_box_pack_start(GTK_BOX(but_box), okbutton, TRUE, TRUE, 2);

	cancelbutton=gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancelbutton), "clicked", GTK_SIGNAL_FUNC(aoout_config_cancel), NULL);
	gtk_box_pack_start(GTK_BOX(but_box), cancelbutton, TRUE, TRUE, 2);

	applybutton=gtk_button_new_with_label("Apply");
	gtk_signal_connect(GTK_OBJECT(applybutton), "clicked", GTK_SIGNAL_FUNC(aoout_config_apply), NULL);
	gtk_box_pack_start(GTK_BOX(but_box), applybutton, TRUE, TRUE, 2);
	
	gtk_widget_show_all(conf_win);
}

GtkWidget *aoout_config_buf_frame(void) {
	GtkWidget *buf_frame, *buf_vbox, *buf_hbox, *buf_hbox2;
	GtkWidget *buf_size_label, *buf_chunk_label;

	buf_frame=gtk_frame_new(_("Buffering"));
	gtk_container_set_border_width(GTK_CONTAINER(buf_frame), 4);

	buf_vbox=gtk_vbox_new(FALSE, 2);
	gtk_container_add(GTK_CONTAINER(buf_frame), buf_vbox);

	buf_en_check=gtk_check_button_new_with_label(_("Enable buffering"));
	gtk_toggle_button_set_active(buf_en_check, ao_cfg.buffering);
	gtk_box_pack_start(GTK_BOX(buf_vbox), buf_en_check, TRUE, TRUE, 2);

	buf_hbox=gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(buf_vbox), buf_hbox, TRUE, TRUE, 2);

	buf_size_spin=gtk_spin_button_new(gtk_adjustment_new(ao_cfg.bufsize, 1, 20000, 1, 500, 0), 0.5, 0);
	gtk_spin_button_set_wrap(buf_size_spin, FALSE);
	gtk_spin_button_set_snap_to_ticks(buf_size_spin, 1);
	gtk_spin_button_set_update_policy(buf_size_spin, GTK_UPDATE_IF_VALID);
	
	buf_size_label=gtk_label_new("Buffer size");
	gtk_label_set_justify(buf_size_label, GTK_JUSTIFY_RIGHT);
	gtk_box_pack_start(GTK_BOX(buf_hbox), buf_size_label, TRUE, TRUE, 8);
	gtk_box_pack_start(GTK_BOX(buf_hbox), buf_size_spin, TRUE, TRUE, 2);

	buf_hbox2=gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(buf_vbox), buf_hbox2, TRUE, TRUE, 2);

	buf_chunk_label=gtk_label_new("Chunk size");
	gtk_label_set_justify(buf_chunk_label, GTK_JUSTIFY_RIGHT);
	gtk_box_pack_start(GTK_BOX(buf_hbox2), buf_chunk_label, TRUE, TRUE, 12);

	buf_chunk_spin=gtk_spin_button_new(gtk_adjustment_new(ao_cfg.bufchunk, 1, 2000, 1, 100, 0), 0.5, 0);
	gtk_spin_button_set_wrap(buf_chunk_spin, FALSE);
	gtk_spin_button_set_snap_to_ticks(buf_chunk_spin, 1);
	gtk_spin_button_set_update_policy(buf_chunk_spin, GTK_UPDATE_IF_VALID);
	gtk_box_pack_start(GTK_BOX(buf_hbox2), buf_chunk_spin, TRUE, TRUE, 2);

	return buf_frame;
}

GtkWidget *aoout_config_file_frame(void) {
	GtkWidget *file_frame, *file_vbox, *file_hbox1, *file_hbox2;
	GtkWidget *file_type_label, *file_dir_label;
	GtkWidget *file_type_omenu, *file_type_menu, *file_type_item;
	GtkWidget *browse_button;

	int num_file_types=3, i, tmp;
	char **filetypes=calloc(num_file_types, sizeof(void *));
	filetypes[0]="wav";
	filetypes[1]="au";
	filetypes[2]="raw";

	file_type=ao_cfg.filetype;

	file_frame=gtk_frame_new(_("File output"));
	gtk_container_set_border_width(GTK_CONTAINER(file_frame), 4);

	file_vbox=gtk_vbox_new(FALSE, 2);
	gtk_container_add(GTK_CONTAINER(file_frame), file_vbox);

	file_en_check=gtk_check_button_new_with_label(_("Enable file output"));
	gtk_toggle_button_set_active(file_en_check, ao_cfg.fileout);
	gtk_box_pack_start(GTK_BOX(file_vbox), file_en_check, TRUE, TRUE, 0);

	file_hbox1=gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(file_vbox), file_hbox1, TRUE, TRUE, 0);

	file_type_label=gtk_label_new("File type to save as");
	gtk_label_set_justify(file_type_label, GTK_JUSTIFY_RIGHT);
	gtk_box_pack_start(GTK_BOX(file_hbox1), file_type_label, TRUE, TRUE, 6);

	file_type_omenu=gtk_option_menu_new();
	gtk_box_pack_start(GTK_BOX(file_hbox1), file_type_omenu, TRUE, TRUE, 2);

	file_type_menu=gtk_menu_new();
	tmp=0;
	for (i=0; i<num_file_types; i++) {
		file_type_item=gtk_menu_item_new_with_label(filetypes[i]);

		gtk_signal_connect(GTK_OBJECT(file_type_item), "activate", aoout_configure_ftsel, (gpointer)filetypes[i]);
		gtk_menu_append(file_type_menu, file_type_item);
		
		if (0==strcmp(filetypes[i], ao_cfg.filetype)) {
			tmp=i;
		}
	}
	gtk_option_menu_set_menu(file_type_omenu, file_type_menu);
	gtk_option_menu_set_history(file_type_omenu, tmp);

	file_hbox2=gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(file_vbox), file_hbox2, TRUE, TRUE, 0);

	file_dir_label=gtk_label_new("Directory to save to");
	gtk_label_set_justify(file_dir_label, GTK_JUSTIFY_RIGHT);
	gtk_box_pack_start(GTK_BOX(file_hbox2), file_dir_label, TRUE, TRUE, 6);

	file_dir_entry=gtk_entry_new();
	gtk_entry_set_text(file_dir_entry, ao_cfg.directory);
	gtk_box_pack_start(GTK_BOX(file_hbox2), file_dir_entry, TRUE, TRUE, 2);

	browse_button=gtk_button_new_with_label("Browse");
	gtk_signal_connect(GTK_OBJECT(browse_button), "clicked", GTK_SIGNAL_FUNC(aoout_config_browse), NULL);
	gtk_box_pack_start(GTK_BOX(file_hbox2), browse_button, TRUE, TRUE, 2);

	file_over_check=gtk_check_button_new_with_label(_("Overwrite existing files"));
	gtk_toggle_button_set_active(file_over_check, ao_cfg.fileoverwrite);
	gtk_box_pack_start(GTK_BOX(file_vbox), file_over_check, TRUE, TRUE, 0);

	return file_frame;
}

GtkWidget *aoout_config_effect_frame(void) {
	GtkWidget *effect_frame, *effect_vbox;

	effect_frame=gtk_frame_new(_("Effects"));
	gtk_container_set_border_width(GTK_CONTAINER(effect_frame), 4);

	effect_vbox=gtk_vbox_new(FALSE, 2);
	gtk_container_add(GTK_CONTAINER(effect_frame), effect_vbox);

	effect_prevol_check=gtk_check_button_new_with_label(_("Apply volume before effects"));
	gtk_toggle_button_set_active(effect_prevol_check, ao_cfg.prevolume);
	gtk_box_pack_start(GTK_BOX(effect_vbox), effect_prevol_check, TRUE, TRUE, 0);

	effect_file_check=gtk_check_button_new_with_label(_("Apply effects when outputting to file"));
	gtk_toggle_button_set_active(effect_file_check, ao_cfg.effectfile);
	gtk_box_pack_start(GTK_BOX(effect_vbox), effect_file_check, TRUE, TRUE, 0);

	return effect_frame;
}

void aoout_config_ok(GtkWidget *widget, gpointer data) {
	aoout_config_apply(NULL, NULL);

	gtk_widget_destroy(conf_win);
}

void aoout_config_apply(GtkWidget *widget, gpointer data) {
	ConfigFile *cfgfile;
	
	if (GTK_TOGGLE_BUTTON(buf_en_check)->active) {
		ao_cfg.buffering=TRUE;
	} else {
		ao_cfg.buffering=FALSE;
	}

	ao_cfg.bufsize=gtk_spin_button_get_value_as_int(buf_size_spin);

	ao_cfg.bufchunk=gtk_spin_button_get_value_as_int(buf_chunk_spin);

	if (GTK_TOGGLE_BUTTON(file_en_check)->active) {
		ao_cfg.fileout=TRUE;
	} else {
		ao_cfg.fileout=FALSE;
	}

	if (GTK_TOGGLE_BUTTON(file_over_check)->active) {
		ao_cfg.fileoverwrite=TRUE;
	} else {
		ao_cfg.fileoverwrite=FALSE;
	}

	if (GTK_TOGGLE_BUTTON(effect_prevol_check)->active) {
		ao_cfg.prevolume=TRUE;
	} else {
		ao_cfg.prevolume=FALSE;
	}

	if (GTK_TOGGLE_BUTTON(effect_file_check)->active) {
		ao_cfg.effectfile=TRUE;
	} else {
		ao_cfg.effectfile=FALSE;
	}

	ao_cfg.directory=gtk_editable_get_chars(GTK_EDITABLE(file_dir_entry), 0, -1);
	ao_cfg.filetype=file_type;

	/*	Save the configuration	*/
	cfgfile=xmms_cfg_open_default_file();

	xmms_cfg_write_boolean(cfgfile, "aoout", "buffering", ao_cfg.buffering);
	xmms_cfg_write_int(cfgfile, "aoout", "buffer_size", ao_cfg.bufsize);
	xmms_cfg_write_int(cfgfile, "aoout", "buffer_chunk", ao_cfg.bufchunk);
	xmms_cfg_write_boolean(cfgfile, "aoout", "file_output", ao_cfg.fileout);
	xmms_cfg_write_string(cfgfile, "aoout", "file_type", ao_cfg.filetype);
	xmms_cfg_write_boolean(cfgfile, "aoout", "file_overwrite", ao_cfg.fileoverwrite);
	xmms_cfg_write_string(cfgfile, "aoout", "file_directory", ao_cfg.directory);
	xmms_cfg_write_boolean(cfgfile, "aoout", "prevolume", ao_cfg.prevolume);
	xmms_cfg_write_boolean(cfgfile, "aoout", "effectfile", ao_cfg.effectfile);
	xmms_cfg_write_default_file(cfgfile);

	xmms_cfg_free(cfgfile);
}

void aoout_config_cancel(GtkWidget *widget, gpointer data) {
	gtk_widget_destroy(conf_win);
}

void aoout_config_browse(GtkWidget *widget, gpointer data) {
	if (!file_browser) {
		file_browser=xmms_create_dir_browser(_("Select the directory the files should be saved to"), ao_cfg.directory, GTK_SELECTION_SINGLE, aoout_config_dirupd8);
		gtk_signal_connect(GTK_OBJECT(file_browser), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &file_browser);
		gtk_window_set_transient_for(GTK_WINDOW(file_browser), GTK_WINDOW(conf_win));
		gtk_widget_show(file_browser);
	}
}

void aoout_config_dirupd8(gchar *dir) {
	gtk_entry_set_text(GTK_WIDGET(file_dir_entry), dir);
}

void aoout_configure_ftsel(GtkWidget *widget, gpointer data) {
	file_type=(char *)data;
}
